"""pypetwalk is a Python library to communicate with the petWALK.control module."""
from __future__ import annotations

import logging
from types import TracebackType

import aiohttp

from pypetwalk.const import (
    API_HTTP_PROTOCOL,
    API_PATH_MAPPING,
    API_STATE_DOOR,
    API_STATE_MAPPING_DOOR_CLOSE,
    API_STATE_MAPPING_DOOR_OPEN,
    API_STATE_MAPPING_SYSTEM_OFF,
    API_STATE_MAPPING_SYSTEM_ON,
    API_STATE_SYSTEM,
)

_LOGGER = logging.getLogger(__name__)


class API:
    """Class for handling local API calls."""

    def __init__(self, host: str, port: int) -> None:
        """Initialize API class."""
        self.server_host = host
        self.server_port = port
        self.session = aiohttp.ClientSession()

    async def __aenter__(self) -> "API":
        """Start API class from context manager."""
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc: BaseException | None,
        traceback: TracebackType | None,
    ) -> None:
        """Stop API class from context manager."""
        await self.close()

    async def close(self) -> None:
        """Wait until all sessions are closed."""
        if self.session:
            await self.session.close()

    async def get_modes(self) -> dict:
        """Get current 'modes' from API."""
        return await self.send_command("mode", None)

    async def set_mode(self, mode: str, value: bool) -> dict:
        """Set new value for given 'mode'."""
        return await self.send_command("mode", {mode: value})

    async def get_states(self) -> dict:
        """Get current 'states' from API."""
        return await self.send_command("state", None)

    async def set_state(self, state: str, value: bool) -> dict:
        """Set new value for given 'state'."""
        if state == API_STATE_SYSTEM:
            new_state = API_STATE_MAPPING_SYSTEM_ON
            if not value:
                new_state = API_STATE_MAPPING_SYSTEM_OFF
        elif state == API_STATE_DOOR:
            new_state = API_STATE_MAPPING_DOOR_CLOSE
            if value:
                new_state = API_STATE_MAPPING_DOOR_OPEN
        return await self.send_command("state", {state: new_state})

    async def send_command(self, command: str, params: dict | None) -> dict:
        """Send command to local API."""
        method = "GET"
        if params:
            method = "PUT"

        url = f"{API_HTTP_PROTOCOL}://{self.server_host}:{self.server_port}\
            {API_PATH_MAPPING[command]}"
        _LOGGER.info("Calling %s with method %s", url, method)
        _LOGGER.debug("... and Parameters: %s", params)
        if method == "GET":
            async with self.session.get(url) as resp:
                if resp.status != 200:
                    error = f"Incorrect status code received {resp.status}"
                    _LOGGER.error(error)
                    return {
                        "status": resp.status,
                        "content": await resp.text(),
                        "error": True,
                    }
                return await resp.json()  # type: ignore[no-any-return]
        elif method == "PUT":
            async with self.session.put(url, json=params) as resp:
                response = {
                    "status": resp.status,
                    "content": "",
                    "error": False,
                }
                if resp.status != 202:  # Currently, API returns only 202
                    response["error"] = True
                    _LOGGER.warning("Incorrect status code %d", resp.status)
                response["content"] = await resp.text()
                return response

        return {}



"""Constants for the petWALK integration."""
from typing import Final

API_PORT: Final = 8080
API_HTTP_PROTOCOL: Final = "http"
WS_PORT: Final = 1234

WS_COMMAND_RFID_START_LEARN: Final = "RFIDStartLearn"
WS_COMMAND_RFID_STOP_LEARN: Final = "RFIDStopLearn"
WS_COMMAND_RFID_DELETE: Final = "RFIDDelete"
WS_COMMAND_RFID_DELETE_ALL: Final = "RFIDDeleteAll"
WS_COMMAND_RFID_DELETE_PET: Final = "RFIDDeletePet"
WS_COMMAND_RFID_TAG_LIST: Final = "RFIDTagList"
WS_COMMAND_RFID_TAG_EXISTS: Final = "RFIDTagExists"
WS_COMMAND_ZIGBEE_LIST_DEVICES: Final = "ZigBeeListDevices"
WS_COMMAND_ZIGBEE_REMOVE_DEVICE: Final = "ZigBeeRemoveDevice"
WS_COMMAND_ZIGBEE_JOIN_ALLOWED: Final = "ZigBeeJoinAllowed"
WS_COMMAND_ZIGBEE_NAME_DEVICE: Final = "ZigBeeNameDevice"
WS_COMMAND_ZIGBEE_UPDATE: Final = "ZigBeeUpdate"
WS_COMMAND_ZIGBEE_JOIN_CONFIRM: Final = "ZigBeeJoinConfirm"
WS_COMMAND_DEVICE_INFO: Final = "DeviceInfo"
WS_COMMAND_WIFI_NETWORK_LIST: Final = "WifiNetworkList"
WS_COMMAND_WIFI_NETWORK_SET: Final = "WifiNetworkSet"
WS_COMMAND_WIFI_SCAN: Final = "WifiScan"
WS_COMMAND_TIME_SET: Final = "TimeSet"
WS_COMMAND_FACTORY_RESET: Final = "FactoryReset"
WS_COMMAND_INIT_DRIVE_START: Final = "InitDriveStart"

ZIGBEE_DEFAULT_JOIN_TYPE: Final = "petWALK_ALB"

API_STATE_BRIGHTNESS_SENSOR: Final = "brightnessSensor"
API_STATE_MOTION_IN: Final = "motion_in"
API_STATE_MOTION_OUT: Final = "motion_out"
API_STATE_RFID: Final = "rfid"
API_STATE_TIME: Final = "time"
API_STATE_DOOR: Final = "door"
API_STATE_SYSTEM: Final = "system"

API_METHOD_MAPPING: dict[str, str] = {
    API_STATE_BRIGHTNESS_SENSOR: "mode",
    API_STATE_MOTION_IN: "mode",
    API_STATE_MOTION_OUT: "mode",
    API_STATE_RFID: "mode",
    API_STATE_TIME: "mode",
    API_STATE_DOOR: "state",
    API_STATE_SYSTEM: "state",
}

API_PATH_MAPPING: dict[str, str] = {"mode": "/modes", "state": "/states"}

API_STATE_MAPPING: dict[str, bool] = {
    "closed": False,
    "close": False,
    "open": True,
    "off": False,
    "on": True,
}

API_STATE_MAPPING_DOOR_OPEN: Final = "open"
API_STATE_MAPPING_DOOR_CLOSE: Final = "close"
API_STATE_MAPPING_DOOR_CLOSED: Final = "closed"
API_STATE_MAPPING_SYSTEM_ON: Final = "on"
API_STATE_MAPPING_SYSTEM_OFF: Final = "off"



"""pypetwalk is a Python library to communicate with the petWALK.control module."""
from __future__ import annotations

import json


class Request:
    """Class to handle Websocket Request Object."""

    def __init__(self) -> None:
        """Initialize Request object."""
        self.data = {"requests": [{"function": "", "params": []}]}

    def build_request(self, command: str, params: list) -> Request:
        """Build request with given parameter."""
        self.data["requests"][0]["function"] = command
        self.data["requests"][0]["params"] = params

        return self

    def get_data(self) -> dict:
        """Return Python dict with current data."""
        return self.data

    def get_json(self) -> str:
        """Return data converted into JSON String."""
        return json.dumps(self.data)
